import configparser
import hashlib
import os
import platform
import random
import tempfile
from typing import Any, Dict, Optional

# download delegate
__wget__ = ("WGET_PATH",)
__transmission__ = (
    "TRANSMISSION_RPC_URL",
    "TRANSMISSION_RPC_PORT",
    "TRANSMISSION_RPC_USERNAME",
    "TRANSMISSION_RPC_PASSWORD",
    "TRANSMISSION_RPC_PATH",
)
__qbittorrent__ = (
    "QBITTORRENT_HOST",
    "QBITTORRENT_PORT",
    "QBITTORRENT_USERNAME",
    "QBITTORRENT_PASSWORD",
    "QBITTORRENT_CATEGORY",
)
__aria2__ = (
    "ARIA2_RPC_URL",
    "ARIA2_RPC_TOKEN",
)
__deluge__ = ("DELUGE_RPC_URL", "DELUGE_RPC_PASSWORD")

__download_delegate__ = __wget__ + __aria2__ + __transmission__ + __deluge__ + __qbittorrent__

# fake __all__
__all__ = (
    "BANGUMI_MOE_URL",
    "SAVE_PATH",
    "DOWNLOAD_DELEGATE",
    "MAX_PAGE",
    "DATA_SOURCE",
    "TMP_PATH",
    "DANMAKU_API_URL",
    "LANG",
    "FRONT_STATIC_PATH",
    "ADMIN_TOKEN",
    "SHARE_DMHY_URL",
    "GLOBAL_FILTER",
    "ENABLE_GLOBAL_FILTER",
    "TORNADO_SERVE_STATIC_FILES",
)

# cannot be rewrite
__readonly__ = (
    "BGMI_PATH",
    "DB_PATH",
    "CONFIG_FILE_PATH",
    "TOOLS_PATH",
    "SCRIPT_PATH",
    "SCRIPT_DB_PATH",
    "FRONT_STATIC_PATH",
)

# writeable
__writeable__ = tuple(i for i in __all__ if i not in __readonly__)

# the real __all__
__all__ = __all__ + __download_delegate__ + __readonly__  # type: ignore

DOWNLOAD_DELEGATE_MAP = {
    "rr!": __wget__,
    "aria2-rpc": __aria2__,
    "transmission-rpc": __transmission__,
    "deluge-rpc": __deluge__,
    "qbittorrent-webapi": __qbittorrent__,
}

if not os.environ.get("BGMI_PATH"):  # pragma: no cover
    if platform.system() == "Windows":
        BGMI_PATH = os.path.join(os.environ.get("USERPROFILE", tempfile.gettempdir()), ".bgmi")
        if not BGMI_PATH:
            raise SystemExit
    else:
        BGMI_PATH = os.path.join(os.environ.get("HOME", "/tmp"), ".bgmi")
else:
    BGMI_PATH = os.environ["BGMI_PATH"]

DB_PATH = os.path.join(BGMI_PATH, "bangumi.db")
CONFIG_FILE_PATH = os.path.join(BGMI_PATH, "bgmi.cfg")

SCRIPT_DB_PATH = os.path.join(BGMI_PATH, "script.db")
SCRIPT_PATH = os.path.join(BGMI_PATH, "scripts")
TOOLS_PATH = os.path.join(BGMI_PATH, "tools")


def read_config() -> None:
    c = configparser.ConfigParser()
    if not os.path.exists(CONFIG_FILE_PATH):
        write_default_config()
        return
    c.read(CONFIG_FILE_PATH, encoding="utf-8")

    for i in __writeable__:
        if c.has_option("bgmi", i):
            v: Any = c.get("bgmi", i)
            if i == "MAX_PAGE":
                v = int(v)
            globals().update({i: v})

    for i in DOWNLOAD_DELEGATE_MAP.get(DOWNLOAD_DELEGATE, []):
        if c.has_option(DOWNLOAD_DELEGATE, i):
            globals().update({i: c.get(DOWNLOAD_DELEGATE, i)})


def print_config() -> Optional[str]:
    c = configparser.ConfigParser()
    if not os.path.exists(CONFIG_FILE_PATH):
        return None

    c.read(CONFIG_FILE_PATH, encoding="utf-8")
    string = ""
    string += "[bgmi]\n"
    for i in __writeable__:
        string += "{}={}\n".format(i, c.get("bgmi", i))

    string += f"\n[{DOWNLOAD_DELEGATE}]\n"
    for i in DOWNLOAD_DELEGATE_MAP.get(DOWNLOAD_DELEGATE, []):
        string += f"{i}={c.get(DOWNLOAD_DELEGATE, i)}\n"
    return string


def write_default_config() -> None:
    c = configparser.ConfigParser()
    if not c.has_section("bgmi"):
        c.add_section("bgmi")

    for k in __writeable__:
        v = globals().get(k, "0")
        if k == "ADMIN_TOKEN" and v is None:
            v = hashlib.md5(str(random.random()).encode("utf-8")).hexdigest()

        c.set("bgmi", k, str(v))

    if not c.has_section(DOWNLOAD_DELEGATE):
        c.add_section(DOWNLOAD_DELEGATE)

    for k in DOWNLOAD_DELEGATE_MAP.get(DOWNLOAD_DELEGATE, []):
        v = globals().get(k, None)
        c.set(DOWNLOAD_DELEGATE, k, v)

    try:
        with open(CONFIG_FILE_PATH, "w+", encoding="utf-8") as f:
            c.write(f)
    except OSError:
        print("[-] Error writing to config file and ignored")


def write_config(config: Optional[str] = None, value: Optional[str] = None) -> Dict[str, Any]:
    if not os.path.exists(CONFIG_FILE_PATH):
        write_default_config()
        return {
            "status": "error",
            "message": "Config file does not exists, writing default config file",
            "data": [],
        }

    c = configparser.ConfigParser()
    c.read(CONFIG_FILE_PATH, encoding="utf-8")
    result = {}  # type: Dict[str, Any]
    try:
        if config is None:
            result = {"status": "info", "message": print_config()}

        elif value is None:  # config(config, None)
            result = {"status": "info"}

            if config in __download_delegate__:
                result["message"] = f"{config}={c.get(DOWNLOAD_DELEGATE, config)}"
            else:
                result["message"] = "{}={}".format(config, c.get("bgmi", config))

        else:  # config(config, Value)
            if config in __writeable__:
                c.set("bgmi", config, value)
                with open(CONFIG_FILE_PATH, "w", encoding="utf-8") as f:
                    c.write(f)

                read_config()
                if config == "DOWNLOAD_DELEGATE" and not c.has_section(DOWNLOAD_DELEGATE):
                    c.add_section(DOWNLOAD_DELEGATE)
                    for i in DOWNLOAD_DELEGATE_MAP.get(DOWNLOAD_DELEGATE, []):
                        v = globals().get(i, "")
                        c.set(DOWNLOAD_DELEGATE, i, v)

                    with open(CONFIG_FILE_PATH, "w", encoding="utf-8") as f:
                        c.write(f)

                result = {
                    "status": "success",
                    "message": f"{config} has been set to {value}",
                }

            elif config in DOWNLOAD_DELEGATE_MAP.get(DOWNLOAD_DELEGATE, []):
                c.set(DOWNLOAD_DELEGATE, config, value)
                with open(CONFIG_FILE_PATH, "w", encoding="utf-8") as f:
                    c.write(f)

                result = {
                    "status": "success",
                    "message": f"{config} has been set to {value}",
                }
            else:
                result = {
                    "status": "error",
                    "message": f"{config} does not exist or not writeable",
                }

    except configparser.NoOptionError:
        write_default_config()
        result = {
            "status": "error",
            "message": "Error in config file, try rerun `bgmi config`",
        }

    result["data"] = [{"writable": True, "name": x, "value": globals()[x]} for x in __writeable__] + [
        {"writable": False, "name": x, "value": globals()[x]} for x in __readonly__
    ]
    return result


# --------- Writeable ---------- #
# Setting bangumi.moe url
BANGUMI_MOE_URL = "https://bangumi.moe"

# Setting share.dmhy.org url
SHARE_DMHY_URL = "https://share.dmhy.org"

# Setting bangumi.moe url
DATA_SOURCE = "bangumi_moe"

# BGmi user path
SAVE_PATH = os.path.join(BGMI_PATH, "bangumi")
FRONT_STATIC_PATH = os.path.join(BGMI_PATH, "front_static")

# admin token
ADMIN_TOKEN = None

# temp path
TMP_PATH = os.path.join(BGMI_PATH, "tmp")

# log path
LOG_PATH = os.path.join(TMP_PATH, "bgmi.log")

# Download delegate
DOWNLOAD_DELEGATE = "aria2-rpc"

# danmaku api url, https://github.com/DIYgod/DPlayer#related-projects
DANMAKU_API_URL = ""

# language
LANG = "zh_cn"

# max page
MAX_PAGE = 3

# aria2
ARIA2_RPC_URL = "http://localhost:6800/rpc"
ARIA2_RPC_TOKEN = "token:"

# deluge
DELUGE_RPC_URL = "http://127.0.0.1:8112/json"
DELUGE_RPC_PASSWORD = "deluge"

# path of wget
WGET_PATH = "/usr/bin/wget"

# transmission-rpc
TRANSMISSION_RPC_URL = "127.0.0.1"
TRANSMISSION_RPC_PORT = "9091"
TRANSMISSION_RPC_USERNAME = "your_username"
TRANSMISSION_RPC_PASSWORD = "your_password"
TRANSMISSION_RPC_PATH = "/transmission/"

# qbittorrent-webapi
QBITTORRENT_HOST = "127.0.0.1"
QBITTORRENT_PORT = "8080"
QBITTORRENT_USERNAME = "admin"
QBITTORRENT_PASSWORD = "adminadmin"
QBITTORRENT_CATEGORY = ""

# tag of bangumi on bangumi.moe
BANGUMI_TAG = "549ef207fe682f7549f1ea90"

# Global blocked keyword
GLOBAL_FILTER = "Leopard-Raws, hevc, x265, c-a Raws, U3-Web"

# enable global filter
ENABLE_GLOBAL_FILTER = "1"

# use tornado serving video files
TORNADO_SERVE_STATIC_FILES = "0"

# ------------------------------ #
# !!! Read config from file and write to globals() !!!
read_config()
# ------------------------------ #
# will be used in other other models
__all_writable_now__ = __writeable__ + DOWNLOAD_DELEGATE_MAP.get(DOWNLOAD_DELEGATE, ())

# --------- Read-Only ---------- #
# platform
IS_WINDOWS = platform.system() == "Windows"

if __name__ == "__main__":
    write_default_config()



import sqlite3

from bgmi.config import BGMI_PATH, DB_PATH, SCRIPT_DB_PATH
from bgmi.utils import print_error

CREATE_TABLE_BANGUMI = """CREATE TABLE IF NOT EXISTS bangumi (
          id INTEGER PRIMARY KEY AUTOINCREMENT,
          name TEXT NOT NULL UNIQUE,
          subtitle_group TEXT NOT NULL,
          keyword TEXT,
          update_time CHAR(5) NOT NULL,
          cover TEXT,
          status INTEGER DEFAULT 0
        )"""

CREATE_TABLE_FOLLOWED = """CREATE TABLE IF NOT EXISTS followed (
          id INTEGER PRIMARY KEY AUTOINCREMENT,
          bangumi_name TEXT NOT NULL UNIQUE,
          episode INTEGER DEFAULT 0,
          status INTEGER DEFAULT 1,
          updated_time INTEGER DEFAULT 0
        )"""

CREATE_TABLE_DOWNLOAD = """CREATE TABLE IF NOT EXISTS download (
          id INTEGER PRIMARY KEY AUTOINCREMENT,
          name TEXT NOT NULL,
          title TEXT NOT NULL,
          episode INTEGER DEFAULT 0,
          download TEXT,
          status INTEGER DEFAULT 0
        )"""

CREATE_TABLE_FOLLOWED_FILTER = """CREATE TABLE IF NOT EXISTS filter (
          id INTEGER PRIMARY KEY  AUTOINCREMENT,
          bangumi_name TEXT  UNIQUE NOT NULL,
          subtitle TEXT,
          include TEXT,
          exclude TEXT,
          regex TEXT
        )"""

CREATE_TABLE_SUBTITLE = """CREATE TABLE IF NOT EXISTS subtitle (
          id TEXT PRIMARY KEY UNIQUE NOT NULL,
          name TEXT
        )"""

CREATE_TABLE_SCRIPT = """CREATE TABLE IF NOT EXISTS scripts (
          id INTEGER PRIMARY KEY  AUTOINCREMENT,
          bangumi_name TEXT UNIQUE NOT NULL,
          episode INTEGER DEFAULT 0,
          status INTEGER DEFAULT 1,
          updated_time INTEGER DEFAULT 0
        )"""

CLEAR_TABLE_ = "DELETE  FROM {}"


def init_db() -> None:
    try:
        # bangumi.db
        conn = sqlite3.connect(DB_PATH)
        conn.execute(CREATE_TABLE_BANGUMI)
        conn.execute(CREATE_TABLE_FOLLOWED)
        conn.execute(CREATE_TABLE_DOWNLOAD)
        conn.execute(CREATE_TABLE_FOLLOWED_FILTER)
        conn.execute(CREATE_TABLE_SUBTITLE)
        conn.commit()
        conn.close()

        # script.db
        conn = sqlite3.connect(SCRIPT_DB_PATH)
        conn.execute(CREATE_TABLE_SCRIPT)
        conn.commit()
        conn.close()
    except sqlite3.OperationalError:
        print_error(f"Open database file failed, path {BGMI_PATH} is not writable.")



from typing import Dict, Type

from bgmi.config import DATA_SOURCE
from bgmi.utils import print_error
from bgmi.website import bangumi_moe, base, mikan, share_dmhy

DATA_SOURCE_MAP: Dict[str, Type[base.BaseWebsite]] = {
    "mikan_project": mikan.Mikanani,
    "bangumi_moe": bangumi_moe.BangumiMoe,
    "dmhy": share_dmhy.DmhySource,
}


try:
    website = DATA_SOURCE_MAP[DATA_SOURCE]()
except KeyError:
    print_error(f'date source "{DATA_SOURCE}" in config is wrong, please edit it manually')



import os
from shutil import copy

from bgmi import __version__
from bgmi.config import BGMI_PATH, FRONT_STATIC_PATH, IS_WINDOWS, SAVE_PATH, SCRIPT_PATH, TMP_PATH, TOOLS_PATH
from bgmi.utils import print_error, print_info, print_success, print_warning


def install_crontab() -> None:
    print_info("Installing crontab job")
    if IS_WINDOWS:
        copy(os.path.join(os.path.dirname(__file__), "others/cron.vbs"), BGMI_PATH)
        os.system(
            'schtasks /Create /SC HOURLY /TN "bgmi updater" /TR "{}"  /IT /F'.format(
                os.path.join(BGMI_PATH, "cron.vbs")
            )
        )
    else:
        path = os.path.join(os.path.dirname(__file__), "others/crontab.sh")
        os.system(f"bash '{path}'")


def create_dir() -> None:
    path_to_create = (
        BGMI_PATH,
        SAVE_PATH,
        TMP_PATH,
        SCRIPT_PATH,
        TOOLS_PATH,
        FRONT_STATIC_PATH,
    )

    if not os.environ.get("HOME", os.environ.get("USERPROFILE", "")):
        print_warning("$HOME not set, use '/tmp/'")

    # bgmi home dir
    try:
        for path in path_to_create:
            if not os.path.exists(path):
                os.makedirs(path)
                print_success(f"{path} created successfully")
        OLD = os.path.join(BGMI_PATH, "old")
        # create OLD if not exist oninstall
        if not os.path.exists(OLD):
            with open(OLD, "w", encoding="utf8") as f:
                f.write(__version__)
    except OSError as e:
        print_error(f"Error: {str(e)}")



import requests
from requests.adapters import HTTPAdapter, Retry

session = requests.Session()

retries = Retry(total=3, backoff_factor=0.1, status_forcelist=[500, 502, 503, 504])
session.mount("https://mikanani.me/", HTTPAdapter(max_retries=retries))



import transmission_rpc

from bgmi import config
from bgmi.plugin.download import BaseDownloadService, DownloadStatus


class TransmissionRPC(BaseDownloadService):
    def __init__(self):
        self.client = transmission_rpc.Client(
            host=config.TRANSMISSION_RPC_URL,
            port=config.TRANSMISSION_RPC_PORT,
            username=config.TRANSMISSION_RPC_USERNAME,
            password=config.TRANSMISSION_RPC_PASSWORD,
            path=config.TRANSMISSION_RPC_PATH,
        )

    @staticmethod
    def check_config() -> None:
        pass

    @staticmethod
    def check_dep():
        pass

    def add_download(self, url: str, save_path: str):
        torrent = self.client.add_torrent(url, download_dir=save_path, paused=False)
        return torrent.hashString

    def get_status(self, id: str) -> DownloadStatus:
        torrent = self.client.get_torrent(id)
        if torrent.error:
            return DownloadStatus.not_found
        return {
            "check pending": DownloadStatus.downloading,
            "checking": DownloadStatus.downloading,
            "downloading": DownloadStatus.downloading,
            "seeding": DownloadStatus.done,
            "stopped": DownloadStatus.not_downloading,
        }.get(torrent.status, DownloadStatus.error)



import os
import sqlite3
from shutil import copy

from bgmi import __version__
from bgmi.config import BGMI_PATH, DB_PATH, SCRIPT_DB_PATH
from bgmi.utils import print_error, print_info

OLD = os.path.join(BGMI_PATH, "old")


def exec_sql(sql: str, db: str = DB_PATH) -> None:
    try:
        print_info(f"Execute {sql}")
        conn = sqlite3.connect(db)
        conn.execute(sql)
        conn.commit()
        conn.close()
    except sqlite3.OperationalError:  # pragma: no cover
        print_error("Execute SQL statement failed", exit_=False)


def update_database() -> None:
    if not os.path.exists(OLD):
        v = "0"
        with open(OLD, "w", encoding="utf8") as f:
            f.write(__version__)
    else:
        with open(OLD, "r+", encoding="utf8") as f:
            v = f.read()
            f.seek(0)
            f.write(__version__)

    if v < "1.0.25":
        exec_sql("ALTER TABLE filter ADD COLUMN regex")

    if v < "1.4.1":
        exec_sql("ALTER TABLE scripts ADD COLUMN update_time INTEGER", SCRIPT_DB_PATH)

    if v < "2.0.6":

        copy(
            os.path.join(os.path.dirname(__file__), "../others/cron.vbs"),
            os.path.join(BGMI_PATH, "cron.vbs"),
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bgmi',
 'bgmi.downloader',
 'bgmi.front',
 'bgmi.lib',
 'bgmi.plugin',
 'bgmi.utils',
 'bgmi.website']

package_data = \
{'': ['*'], 'bgmi': ['others/*'], 'bgmi.front': ['templates/*']}

install_requires = \
['beautifulsoup4==4.11.1',
 'filetype==1.2.0',
 'icalendar==5.0.4',
 'peewee==3.15.4',
 'pydantic==1.10.4',
 'qbittorrent-api==2022.11.40',
 'requests==2.28.2',
 'stevedore==4.1.1',
 'tornado==6.2',
 'transmission-rpc==3.4.0',
 'wcwidth>=0.2.5,<0.3.0']

entry_points = \
{'bgmi.downloader': ['aria2-rpc = bgmi.downloader:Aria2DownloadRPC',
                     'deluge-rpc = bgmi.downloader:DelugeRPC',
                     'qbittorrent-webapi = bgmi.downloader:QBittorrentWebAPI',
                     'transmission-rpc = bgmi.downloader:TransmissionRPC'],
 'console_scripts': ['bgmi = bgmi.main:main',
                     'bgmi_http = bgmi.front.server:main']}

setup_kwargs = {
    'name': 'bgmi',
    'version': '2.2.18',
    'description': 'BGmi is a cli tool for subscribed bangumi.',
    'long_description': '# BGmi\n\nBGmi is a cli tool for subscribed bangumi.\n\n[中文说明](./README.cn.md)\n\n[![pypi](https://img.shields.io/pypi/v/bgmi.svg)](https://pypi.python.org/pypi/bgmi)\n[![download](https://pepy.tech/badge/bgmi/month)](https://pepy.tech/project/bgmi)\n[![test](https://github.com/BGmi/BGmi/actions/workflows/test.yaml/badge.svg)](https://github.com/BGmi/BGmi/actions/workflows/test.yaml)\n[![coverage](https://codecov.io/gh/BGmi/BGmi/branch/master/graph/badge.svg)](https://codecov.io/gh/BGmi/BGmi)\n[![license](https://img.shields.io/badge/License-MIT-blue.svg)](https://github.com/BGmi/BGmi/blob/master/LICENSE)\n\n## TODO\n\n## Update Log\n\n- Remove Python3.7 support\n- Remove Python3.6 support as it has reached its end-of-life\n- [Allow adding new download delegate without modifying the source code](./docs/downloader.md)\n- Remove xunlei-lixian support\n- New download delegate [qbittorrent-webapi](https://www.qbittorrent.org/)\n- Remove Python2 support\n- Transmission RPC authentication configuration\n- New download delegate [deluge-rpc](https://www.deluge-torrent.org/)\n- You can filter search results by min and max episode\n\nmore details can be found at [releases](https://github.com/BGmi/BGmi/releases) and [unreleased changelog](https://github.com/BGmi/BGmi/issues/297)\n\n## Feature\n\n- Multi data sources supported: [bangumi_moe](https://bangumi.moe), [mikan_project](https://mikanani.me) or [dmhy](https://share.dmhy.org/)\n- Use aria2, transmission, deluge or qbittorrent to download bangumi\n- Web interface to manage bangumi with HTTP API\n- Play bangumi online with danmaku\n- RSS feed for uTorrent, ICS calendar for mobile devices\n- Bangumi Script: Write your own bangumi parser!\n- Bangumi calendar / episode information\n- Keyword, subtitle group, regular expression filters for download bangumi\n- Windows, Linux and Router system supported, BGmi everywhere\n\n![image](./images/bgmi_cli.png?raw=true%0A%20:alt:%20BGmi%0A%20:align:%20center)\n\n![image](./images/bgmi_http.png?raw=true%0A%20:alt:%20BGmi%20HTTP%20Service%0A%20:align:%20center)\n\n![image](./images/bgmi_player.png?raw=true%0A%20:alt:%20BGmi%20HTTP%20Service%0A%20:align:%20center)\n\n![image](./images/bgmi_admin.png?raw=true%0A%20:alt:%20BGmi%20HTTP%20Service%0A%20:align:%20center)\n\n## Installation\n\nUsing pip:\n\n```bash\npip install bgmi\n```\n\nOr from source(not recommended):\n\n```bash\ngit clone https://github.com/BGmi/BGmi\ncd BGmi\npython -m pip install -U pip\npip install .\n```\n\nInit BGmi database and install BGmi web interface:\n\n```bash\nbgmi install\n```\n\n## Upgrade\n\n```bash\npip install bgmi -U\nbgmi upgrade\n```\n\nMake sure to run bgmi upgrade after you upgrade your bgmi\n\n## Docker\n\ngo to [BGmi/bgmi-docker-all-in-one](https://github.com/BGmi/bgmi-docker-all-in-one)\n\n## Usage of bgmi\n\nCli completion(bash and zsh. Shell was detected from your env \\$SHELL)\n\n```bash\neval "$(bgmi complete)"\n```\n\nIf you want to setup a custom BGMI_PATH instead of default `$HOME/.bgmi`:\n\n```bash\nBGMI_PATH=/bgmi bgmi -h\n```\n\nOr add this code to your .bashrc file:\n\n```bash\nalias bgmi=\'BGMI_PATH=/tmp bgmi\'\n```\n\nSupported data source:\n\n- [bangumi_moe(default)](https://bangumi.moe)\n- [mikan_project](https://mikanani.me)\n- [dmhy](https://share.dmhy.org/)\n\n### Help\n\nyou can add `--help` to all `BGmi` sub command to show full options, some of them are not mentioned here.\n\n### Change data source\n\n**All bangumi info in database will be deleted when changing data source!** but scripts won\'t be affected\n\nvideo files will still be stored on the disk, but won\'t be shown on website.\n\n```console\nbgmi source mikan_project\n```\n\n### Show bangumi calendar\n\n```bash\nbgmi cal\n```\n\n### Subscribe bangumi\n\n```bash\nbgmi add "Re:CREATORS" "夏目友人帐 陆" "进击的巨人 season 2"\nbgmi add "樱花任务" --episode 0\n```\n\nDefault episode will be the latest episode.\nIf you just add a bangumi that you haven\'t watched any episodes, considering `bgmi add $BANGUMI_NAME --episode 0`.\n\n### Unsubscribe bangumi\n\n```bash\nbgmi delete --name "Re:CREATORS"\n```\n\n### Update bangumi\n\nUpdate bangumi database (which locates at \\~/.bgmi/bangumi.db acquiescently):\n\n```bash\nbgmi update --download # download all undownloaded episode fo all followed bangumi\nbgmi update "从零开始的魔法书" --download 2 3 # will download specific episide 2 and 3\nbgmi update "时钟机关之星" --download # will download all undownloaded episode for specific bangumi\n```\n\n### Filter download\n\nSet up the bangumi subtitle group filter and fetch entries:\n\n```bash\nbgmi list\nbgmi fetch "Re:CREATORS"\n# include and exclude are case insensitive.\n# --include \'720p\' is equal to --include \'720P\'\nbgmi filter "Re:CREATORS" --subtitle "DHR動研字幕組,豌豆字幕组" --include 720P --exclude BIG5\nbgmi fetch "Re:CREATORS"\n# remove subtitle, include and exclude keyword filter and add regex filter.\nbgmi filter "Re:CREATORS" --subtitle "" --include "" --exclude ""\nbgmi filter "Re:CREATORS" --regex "(DHR動研字幕組|豌豆字幕组).*(720P)"\nbgmi fetch "Re:CREATORS"\n```\n\n#### Global Filter (exclude)\n\nThese words are pre-set as global filter (exclude keywords) `Leopard-Raws`, `hevc`, `x265`, `c-a Raws`, `U3-Web`.\n\nYou can disable global filter with `bgmi config ENABLE_GLOBAL_FILTER 0`.\n\nOr change global filter with `bgmi config GLOBAL_FILTER "Leopard-Raws, hevc, x265, c-a Raws, U3-Web"`\n\n### Search episodes\n\n```bash\nbgmi search \'为美好的世界献上祝福！\' --regex-filter \'.*动漫国字幕组.*为美好的世界献上祝福！.*720P.*\'\n# download\nbgmi search \'为美好的世界献上祝福！\' --regex-filter \'.*合集.*\' --download\n```\n\n### Modify downloaded bangumi episode\n\n```bash\nbgmi list\nbgmi mark "Re:CREATORS" 1\n```\n\nThis will tell bgmi to not need to download episode less than or equal to 1.\n\n### Manage download items\n\n```bash\nbgmi download --list\nbgmi download --list --status 0\nbgmi download --mark 1 --status 2\n```\n\nStatus code:\n\n- 0 - Not downloaded items\n- 1 - Downloading items\n- 2 - Downloaded items\n\n### Show BGmi configure and modify it\n\n```bash\nbgmi config\nbgmi config ARIA2_RPC_TOKEN \'token:token233\'\n```\n\nFields of configure file:\n\nBGmi configure:\n\n- `BANGUMI_MOE_URL`: url of bangumi.moe mirror\n- `SAVE_PATH`: bangumi saving path\n- `DOWNLOAD_DELEGATE`: the ways of downloading bangumi (aria2-rpc, transmission-rpc, deluge-rpc, qbittorrent-webapi)\n- `MAX_PAGE`: max page for fetching bangumi information\n- `TMP_PATH`: just a temporary path\n- `DANMAKU_API_URL`: url of danmaku api\n- `LANG`: language\n\nAria2-rpc configure:\n\n- `ARIA2_RPC_URL`: aria2c daemon RPC url, not jsonrpc url.("<http://localhost:6800/rpc>" for localhost)\n- `ARIA2_RPC_TOKEN`: aria2c daemon RPC token("token:" for no token)\n\nTransmission-rpc configure:\n\n- `TRANSMISSION_RPC_URL`: transmission rpc host\n- `TRANSMISSION_RPC_PORT`: transmission rpc port\n- `TRANSMISSION_RPC_USERNAME`: transmission rpc username (leave it default if you don\'t set rpc authentication)\n- `TRANSMISSION_RPC_PASSWORD`: transmission rpc password (leave it default if you don\'t set rpc authentication)\n- `TRANSMISSION_RPC_PATH`: transmission rpc path (Corresponding to `"rpc-url": "/transmission/",` in the transmission configuration file)\n\nDeluge-rpc configure:\n\n- `DELUGE_RPC_URL`: deluge rpc url\n- `DELUGE_RPC_PASSWORD`: deluge rpc password\n\nqbittorrent-webapi configure:\n\n- `QBITTORRENT_HOST`: qbittorrent WebAPI host\n- `QBITTORRENT_PORT`: qbittorrent WebAPI port\n- `QBITTORRENT_USERNAME`: qbittorrent WebUI username\n- `QBITTORRENT_PASSWORD`: qbittorrent WebUI password\n- `QBITTORRENT_CATEGORY`: qbittorrent new task category (leave it default if you don\'t need to set category)\n\n### Usage of bgmi_http\n\nDownload all bangumi cover first:\n\n```bash\nbgmi cal --download-cover\n```\n\nDownload frontend static files(you may have done it before):\n\n```bash\nbgmi install\n```\n\nStart BGmi HTTP Service bind on 0.0.0.0:8888:\n\n```bash\nbgmi_http --port=8888 --address=0.0.0.0\n```\n\n### Use bgmi_http on Windows\n\nJust start your bgmi_http and open [<http://localhost:8888/>](http://localhost:8888/) in your browser.\n\nConsider most people won\'t use Nginx on Windows, bgmi_http use tornado.web.StaticFileHandler to serve static files(frontend, bangumi covers, bangumi files) without Nginx.\n\n### Use bgmi_http on Linux\n\nGenerate Nginx config\n\n```bash\nbgmi gen nginx.conf --server-name bgmi.whatever.com > bgmi.whatever.com\n```\n\nOr write your config file manually.\n\n```nginx\nserver {\n    listen 80;\n    server_name bgmi;\n\n    root /path/to/bgmi;\n    autoindex on;\n    charset utf-8;\n\n    location /bangumi {\n        # ~/.bgmi/bangumi\n        alias /path/to/bangumi;\n    }\n\n    location /api {\n        proxy_pass http://127.0.0.1:8888;\n        # Requests to api/update may take more than 60s\n        proxy_connect_timeout 500s;\n        proxy_read_timeout 500s;\n        proxy_send_timeout 500s;\n    }\n\n    location /resource {\n        proxy_pass http://127.0.0.1:8888;\n    }\n\n    location / {\n        # ~/.bgmi/front_static/;\n        alias /path/to/front_static/;\n    }\n\n}\n```\n\nOf cause you can use [yaaw](https://github.com/binux/yaaw/) to manage download items if you use aria2c to download bangumi.\n\n```nginx\n...\nlocation /yaaw {\n    alias /path/to/yaaw;\n}\n\nlocation /jsonrpc {\n    # aria2c rpc\n    proxy_pass http://127.0.0.1:6800;\n}\n...\n```\n\nExample file: [bgmi.conf](https://github.com/BGmi/BGmi/blob/dev/bgmi.conf)\n\n#### DPlayer and Danmaku\n\nBGmi use [DPlayer](https://github.com/DIYgod/DPlayer) to play bangumi.\n\nFirst, setup nginx to access bangumi files. Second, choose one danmaku backend at [DPlayer\\#related-projects](https://github.com/DIYgod/DPlayer#related-projects).\n\nUse bgmi config to setup the url of danmaku api.\n\n```bash\nbgmi config DANMAKU_API_URL https://api.prprpr.me/dplayer/ # This api is provided by dplayer official\n```\n\n...restart your bgmi_http and enjoy :D\n\n#### macOS launchctl service controller\n\nsee [issue \\#77](https://github.com/BGmi/BGmi/pull/77)\n\n[me.ricterz.bgmi.plist](https://github.com/BGmi/BGmi/blob/master/bgmi/others/me.ricterz.bgmi.plist)\n\n## Bangumi Script\n\nBangumi Script is a script which you can write the bangumi parser own. BGmi will load the script and call the method you write before the native functionality.\n\nBangumi Script Runner will catch the data you returned, update the database, and download the bangumi. You only just write the parser and return the data.\n\nBangumi Script is located at BGMI_PATH/script, inherited ScriptBase class.\n\nexamples: [script_example.py](./script_example.py)\n\n`get_download_url` returns a dict as follows.\n\n```python\n{\n    1: \'http://example.com/Bangumi/1/1.torrent\',\n    2: \'http://example.com/Bangumi/1/2.torrent\',\n    3: \'http://example.com/Bangumi/1/3.torrent\'\n}\n```\n\nThe keys 1, 2, 3 is the episode, the value is the url of bangumi, make sure your download delegate support it..\n\n## BGmi Data Source\n\nYou can easily add your own BGmi data source by extending BGmi website base class and implement all the method.\n\n```python\nfrom typing import List, Optional\n\nfrom bgmi.website.base import BaseWebsite\nfrom bgmi.website.model import WebsiteBangumi, Episode\n\n\nclass DataSource(BaseWebsite):\n    def search_by_keyword(\n        self, keyword: str, count: int\n    ) -> List[Episode]:  # pragma: no cover\n        """\n\n        :param keyword: search key word\n        :param count: how many page to fetch from website\n        :return: list of episode search result\n        """\n        raise NotImplementedError\n\n    def fetch_bangumi_calendar(self,) -> List[WebsiteBangumi]:  # pragma: no cover\n        """\n        return a list of all bangumi and a list of all subtitle group\n\n        list of bangumi dict:\n        update time should be one of [\'Sun\', \'Mon\', \'Tue\', \'Wed\', \'Thu\', \'Fri\', \'Sat\', \'Unknown\']\n        """\n        raise NotImplementedError\n\n    def fetch_episode_of_bangumi(\n        self, bangumi_id: str, max_page: int, subtitle_list: Optional[List[str]] = None\n    ) -> List[Episode]:  # pragma: no cover\n        """\n        get all episode by bangumi id\n\n        :param bangumi_id: bangumi_id\n        :param subtitle_list: list of subtitle group\n        :type subtitle_list: list\n        :param max_page: how many page you want to crawl if there is no subtitle list\n        :type max_page: int\n        :return: list of bangumi\n        """\n        raise NotImplementedError\n\n\n    def fetch_single_bangumi(self, bangumi_id: str) -> WebsiteBangumi:\n        """\n        fetch bangumi info when updating\n\n        :param bangumi_id: bangumi_id, or bangumi[\'keyword\']\n        :type bangumi_id: str\n        :rtype: WebsiteBangumi\n        """\n        # return WebsiteBangumi(keyword=bangumi_id) if website don\'t has a page contains episodes and info\n```\n\n## Debug\n\nSet env BGMI_LOG to debug, info, warning, error for different log level\n\nlog file will locate at {TMP_PATH}/bgmi.log\n\n## Uninstall\n\nScheduled task will not be delete automatically, you will have to remove them manually.\n\n`*nix`:\n\nremove them from your crontab\n\n`windows`:\n\n```powershell\nschtasks /Delete /TN \'bgmi updater\'\n```\n\n## License\n\n[MIT License](https://github.com/BGmi/BGmi/blob/master/LICENSE)\n',
    'author': 'RicterZ',
    'author_email': 'ricterzheng@gmail.com',
    'maintainer': 'Trim21',
    'maintainer_email': 'i@trim21.me',
    'url': 'https://github.com/BGmi/BGmi',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.0,<4.0.0',
}


setup(**setup_kwargs)
